/**
 * Program Name: createScript.java
 * Comments:
 * This is a parent class for all of the createJob scripts for the different
 * platforms.  It receives the ContextManager object from the calling JSP page
 * (or whatever).  This object is protected, and can be accesses by the 
 * children.  
 * 
 * We want to write the script to the user's "Descriptors" directory, which 
 * is located under the WEB-INF directory.  It is thus not accessible by
 * the web server and thus by the JSP pages.  So we have to use a bean.
 *
 * The String scriptString is used to store the script as it is constructed.
 * Part of this occurs here (the createCommands() method) and part must 
 * occur in the appropriate child (the createHeader() method).  So this 
 * string is protected and inherited by the child classes.
 *
 * I am assuming the following naming convention for the ContextData:
 * Directory: the user's decriptors directory (xml).
 * IOStyle: type of I/O.  Can by C-style, standard I/O, or internal(xml).
 * WorkDir: name of the workspace directory in which the program will
 *          execute (xml).
 * Name: name of problem (user).
 * NumInParams: number of input parameters (xml).
 * NumOutFiles: number of output files (xml).
 * inputParam: values of the input parameters, include input filenames.
 *             Possibly multivalues. (user or xml).
 * outputFiles: names of the output files.  Possible multivalued (user or xml).
 * execpath: full path of the executable (xml).
 * 
 * The data source can either be from ApplDesc.xml (xml) or from the 
 * user (user).
 */

package extensions.gem;

import WebFlow.*;
import WebFlow.ContextManager.*;
import java.io.*;
import java.util.*;

public abstract class createScript {
    protected ContextManager cm;
    protected ContextManagerBean my_cm;
    protected String descDir;
    protected PrintWriter pw;
    protected String scriptString=null;
    //This needs to be put into the application description.
    String MonServUrl="https://claw.arl.hpc.mil:8443/GOW/servlet/WebFlow.Servlets.MonitorJobServ";
    String userName;
    String javaCommand="/usr/java/bin/java ";
    
    moduleServerBean msb;
    Hashtable ch;

    public createScript() { }
   
    public createScript(ContextManagerBean cm,String name) {
	my_cm=cm;
	userName=name;
	descDir=cm.getCurrentSessionProperty("Directory");
	try {
	    pw=new PrintWriter(new FileWriter(descDir+"/job.script"));
	}
	catch(Exception ex) {
	    System.err.println("Could not open file for printing");
	    ex.printStackTrace();
	}
    }

    public createScript(moduleServerBean msb, String name) {
	this.msb=msb;
	cm=msb.getContextManager();
	userName=name;
	descDir=cm.getCurrentSessionProperty("Directory");
	try {
	    pw=new PrintWriter(new FileWriter(descDir+"/job.script"));
	}
	catch(Exception ex) {
	    System.err.println("Could not open file for printing");
	    ex.printStackTrace();
	}
    }

    public createScript(ContextManager cm, String name) {
	this.cm=cm;
	userName=name;
	descDir=cm.getCurrentSessionProperty("Directory");
	try {
	    pw=new PrintWriter(new FileWriter(descDir+"/job.script"));
	}
	catch(Exception ex) {
	    System.err.println("Could not open file for printing");
	    ex.printStackTrace();
	}
    }

    /** 
     * The child should construct the script as a single string and
     * use the following method to print it out.
     */
    public void writeScript() {
	pw.print(scriptString);
	pw.flush();
	pw.close();
    }

    /**
     * The following method should be overriden by each child.  This
     * creates a tag that associates each flag with its name for a 
     * given queuing system.
     */
    public abstract void createFlagTable();

    /**
     * The following method is used to construct the header information
     * with the queue specific directives.  For example, a PBS script
     * begins with
     *   #PBS -q default
     *   #PBS -l walltime=10:00:00
     * and so on.  Must be overriden by each child.
     */
    public abstract void createHeader();

    /**
     * The following method should be used to create the command portion
     * of the job script.  This part is code specific, not queue specific,
     * so we can do this here.
     * 
     * Note this requires that the ContextData has the following properties
     * set: IOStylem, WorkDir, Name.  This is FRAGILE since changes in 
     * the JSP pages that collect info and store them in the ContextData can
     * potentially break this method.  Currently we have to really on 
     * a name convention.  We have no way of enforcing this convention.  
     * Probably should move to XML/DTD for this reason.
     */
    public void createCommands() {
	String IOStyle=my_cm.getCurrentSessionProperty("IOStyle");
	String workDir=my_cm.getCurrentSessionProperty("WorkDir");
	String jobName=my_cm.getCurrentProblemProperty("Name")
	    +"."+my_cm.getCurrentSessionProperty("Name");
	String[] inputParams = null;
	String[] inputFiles = null;
	String[] outputFiles = null;
	String[] outputParams = null;

	String numinparams = my_cm.getCurrentSessionProperty("NumberOfInParams");
	String numinfiles = my_cm.getCurrentSessionProperty("NumberOfInFiles");
	String numoutfiles = my_cm.getCurrentSessionProperty("NumberOfOutFiles");
	String numoutparams = my_cm.getCurrentSessionProperty("NumberOfOutParams");

	int inparamArg = Integer.parseInt(numinparams);
	int infileArg = Integer.parseInt(numinfiles);
	int outfileArg = Integer.parseInt(numoutfiles);
	int outparamArg = Integer.parseInt(numoutparams);

	String inSeparator="",outSeparator="";
	
	if (inparamArg != 0) inputParams=parseMultiVal("inputParam",inparamArg);
	if (infileArg != 0) inputFiles=parseMultiVal("inputFiles",infileArg);
	if (outfileArg != 0) outputFiles=parseMultiVal("outputFiles",outfileArg);
	if (outparamArg != 0) outputParams=parseMultiVal("outputParams",outparamArg);

	scriptString+="setenv CLASSPATH "
	    +my_cm.getCurrentSessionProperty("CLASSPATH")+"\n";

	//Put in event notification
	scriptString+="##DO NOT DELETE THE FOLLOWING LINE\n";
	scriptString+=javaCommand+" EventGenerator "
	    +MonServUrl+" "+userName+" "
	    +my_cm.getCurrentProblemProperty("Name")
	    +"."+my_cm.getCurrentSessionProperty("Name")
	    +" running"
	    +" "+my_cm.getCurrentSessionProperty("machineName")+"\n\n";

	if(IOStyle.equals("CArgument")){
	    inSeparator=" ";
	    outSeparator=" ";
	}
	else if(IOStyle.equals("StandardIO")){
	    inSeparator=" < ";
	    outSeparator=" > ";
	}
	else if(IOStyle.equals("Internal")){
	}
	else {
	    System.err.println("Invalid IOStyle detected");
	}

	scriptString+=my_cm.getCurrentSessionProperty("ExecPath");
 
	for(int i=0;i<infileArg;i++)
	    scriptString=scriptString+inSeparator+inputFiles[i];
 
	for(int j=0;j<outfileArg;j++) 
	    scriptString=scriptString+outSeparator+outputFiles[j];
	
	scriptString+="\n\n";

	scriptString+="##DO NOT DELETE THE FOLLOWING LINE\n";
	scriptString+=javaCommand+" EventGenerator "
	    +MonServUrl+" "+userName+" "
	    +my_cm.getCurrentProblemProperty("Name")
	    +"."+my_cm.getCurrentSessionProperty("Name")+" finished"
            +" "+my_cm.getCurrentSessionProperty("machineName")+"\n\n";

	scriptString+="\n";
    }   
    
    /**
     * This version of createCommands uses the hashtable instead 
     * of the context manager.
     */ 
    public void createHashCommands() {
	String IOStyle=(String)ch.get("IOStyle");
	String workDir=(String)ch.get("WorkDir");
	String jobName=(String)ch.get("ProblemName")
	    +"."+(String)ch.get("SessionName");
	String[] inputParams = null;
	String[] inputFiles = null;
	String[] outputFiles = null;
	String[] outputParams = null;

	String numinparams = (String)ch.get("NumberOfInParams");
	String numinfiles = (String)ch.get("NumberOfInFiles");
	String numoutfiles = (String)ch.get("NumberOfOutFiles");
	String numoutparams = (String)ch.get("NumberOfOutParams");

	int inparamArg = Integer.parseInt(numinparams);
	int infileArg = Integer.parseInt(numinfiles);
	int outfileArg = Integer.parseInt(numoutfiles);
	int outparamArg = Integer.parseInt(numoutparams);

	String inSeparator="",outSeparator="";
	
	if (inparamArg != 0) inputParams=parseMultiValHash("inputParam",inparamArg);
	if (infileArg != 0) inputFiles=parseMultiValHash("inputFiles",infileArg);
	if (outfileArg != 0) outputFiles=parseMultiValHash("outputFiles",outfileArg);
	if (outparamArg != 0) outputParams=parseMultiValHash("outputParams",outparamArg);

	scriptString+="setenv CLASSPATH "
	    +(String)ch.get("CLASSPATH")+"\n";

	//Set environment variables.
	String tmp=(String)ch.get("Environment");
	Vector[] multivec=parseMultiVal2(tmp);
	int size=multivec[0].size();   //both vectors are the same size
	for(int i=0;i<size;i++) {
	    scriptString+="setenv "+(String)multivec[0].get(i)
		+" "+(String)multivec[1].get(i)+"\n";
	}
	
	scriptString+="\n";

	//Switch to working directory in scratch
	scriptString+="cd $SCR\n\n";

	//Put in event notification
	scriptString+="##DO NOT DELETE THE FOLLOWING LINE\n";
	scriptString+=javaCommand+" EventGenerator "
	    +MonServUrl+" "+userName+" "
	    +(String)ch.get("ProblemName")
	    +"."+(String)ch.get("SessionName")
	    +" running"
	    +" "+(String)ch.get("MachineName")+"\n\n";

	if(IOStyle.equals("CArgument")){
	    inSeparator=" ";
	    outSeparator=" ";
	}
	else if(IOStyle.equals("StandardIO")){
	    inSeparator=" < ";
	    outSeparator=" > ";
	}
	else if(IOStyle.equals("Internal")){
	}
	else {
	    System.err.println("Invalid IOStyle detected");
	}

	scriptString+=(String)ch.get("execpath");
 
	for(int i=0;i<infileArg;i++)
	    scriptString=scriptString+inSeparator+inputFiles[i];
 
	for(int j=0;j<outfileArg;j++) 
	    scriptString=scriptString+outSeparator+outputFiles[j];
	
	scriptString+="\n\n";

	scriptString+="##DO NOT DELETE THE FOLLOWING LINE\n";
	scriptString+=javaCommand+" EventGenerator "
	    +MonServUrl+" "+userName+" "
	    +(String)ch.get("ProblemName")
	    +"."+(String)ch.get("SessionName")+" finished"
            +" "+(String)ch.get("MachineName")+"\n\n";

	scriptString+="\n";
	scriptString+="#Move files to mass storage\n";
	scriptString+="msscmd -b \"tar cf "+jobName+".tar *\"\n";
    }   
    
    /**
     * ContextData is basically a hashtable, but often we need to store
     * multiple values for a single name.  For example, if we have multiple
     * input files, we would like to store them under a single name and 
     * restore them in an array.  
     * 
     * We will adopt the convention (following the way child contexts are
     * handled) by separating multiple values with a "+" sign.  For example,
     * we may have
     *             inputParam infile1+infile2+infile3
     */
    private String[] parseMultiVal(String cdParamName,int dim){
	String toEat=my_cm.getCurrentSessionProperty(cdParamName);
	String[] multiVal=new String[dim];
	StringTokenizer st=new StringTokenizer(toEat,"+");
	for(int i=0;i<dim;i++) {
	    multiVal[i]=st.nextToken();
	}
	return multiVal;
    }

    /**
     * ContextData is basically a hashtable, but often we need to store
     * multiple values for a single name.  For example, if we have multiple
     * input files, we would like to store them under a single name and 
     * restore them in an array.  
     * 
     * We will adopt the convention (following the way child contexts are
     * handled) by separating multiple values with a "+" sign.  For example,
     * we may have
     *             inputParam infile1+infile2+infile3
     */
    private String[] parseMultiValHash(String cdParamName,int dim){
	String toEat=(String)ch.get(cdParamName);
	String[] multiVal=new String[dim];
	StringTokenizer st=new StringTokenizer(toEat,"+");
	for(int i=0;i<dim;i++) {
	    multiVal[i]=st.nextToken();
	}
	return multiVal;
    }

    /**
     * Returns a two-dimensional array of vectors.  The first vector
     * contains the names of environment variables, the second the
     * values.
     */
    private Vector[] parseMultiVal2(String catString) {
	Vector[] returnvec=new Vector[2];
	returnvec[0]=new Vector();
	returnvec[1]=new Vector();
	StringTokenizer st=new StringTokenizer(catString,"+");
	while(st.hasMoreTokens()){
	    returnvec[0].add(st.nextToken());
	    returnvec[1].add(st.nextToken());
	}
	return returnvec;
    }
}




